'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var path = _interopDefault(require('path'));
var Metalsmith = _interopDefault(require('metalsmith'));
var async = _interopDefault(require('async'));
var consolidate = _interopDefault(require('consolidate'));

var render = consolidate.handlebars.render;

function gracefulCopy(src, dest, ref) {
  if ( ref === void 0 ) ref = {};
  var data = ref.data;
  var cwd = ref.cwd; if ( cwd === void 0 ) cwd = process.cwd();
  var clean = ref.clean; if ( clean === void 0 ) clean = true;

  return new Promise(function (resolve, reject) {
    var source = path.resolve(cwd, src);
    Metalsmith(source) // eslint-disable-line new-cap
      .source('.')
      .use(template)
      .clean(clean)
      .destination(path.resolve(source, '../', dest))
      .build(function (err, files) {
        if (err) { return reject(err) }
        resolve(Object.keys(files));
      });
  })

  function template(files, metalsmith, done) {
    var keys = Object.keys(files);

    async.each(keys, run, done);

    function run(file, done) {
      var str = files[file].contents.toString();
      // do not attempt to render files that do not have mustaches
      if (!/{{([^{}]+)}}/g.test(str)) {
        return done()
      }
      render(str, data, function (err, res) {
        if (err) {
          return done(err)
        }
        files[file].contents = new Buffer(res);
        done();
      });
    }
  }
}

module.exports = gracefulCopy;
